#
# DNS Balance
#
# アドレスデータベース
#
# By: YOKOTA Hiroshi <yokota@netlab.is.tsukuba.ac.jp>

# $Id: addrdb.rb,v 1.2 2000/08/08 04:19:36 elca Exp $

$addr_db = {
  "default" => {
    "localhost" => [
      [[127,0,0,1], 0],
    ],
  },
}
